-- Author: VertexFloat
-- Date: 20.03.2022
-- Version: Farming Simulator 22, 1.0.0.0
-- Copyright (C): VertexFloat, All Rights Reserved
-- BewareOfStraw main function

BewareOfStraw = {}

function BewareOfStraw.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "getIsCollectingStraw", Cutter.getIsCollectingStraw)
end

function BewareOfStraw:addCutterArea(superFunc, area, realArea, inputFruitType, outputFillType, strawRatio, strawGroundType, farmId, cutterLoad)
	local spec = self.spec_combine
	
	if area > 0 and (spec.lastCuttersFruitType == FruitType.UNKNOWN or spec.lastCuttersArea == 0 or spec.lastCuttersOutputFillType == outputFillType) then
		spec.lastCuttersArea = spec.lastCuttersArea + area
		spec.lastCuttersOutputFillType = outputFillType
		spec.lastCuttersInputFruitType = inputFruitType
		spec.lastCuttersAreaTime = g_currentMission.time
		
		if not spec.swath.isAvailable then
			local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(inputFruitType)
			
			spec.isSwathActive = not fruitDesc.hasWindrow
		end
		
		local litersPerSqm = 60
		local fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(inputFruitType)
		
		if fruitDesc.windrowLiterPerSqm ~= nil then
			litersPerSqm = fruitDesc.windrowLiterPerSqm
		end
		
		if self:getIsThreshingDuringRain() then
			realArea = realArea * (1 - Combine.RAIN_YIELD_REDUCTION)
		end
		
		if self:getFillUnitLastValidFillType(spec.fillUnitIndex) == outputFillType or self:getFillUnitLastValidFillType(spec.bufferFillUnitIndex) == outputFillType then
			local liters = realArea * g_currentMission:getFruitPixelsToSqm() * litersPerSqm * strawRatio
			
			if liters > 0 then
				local inputBuffer = spec.processing.inputBuffer
				local slot = inputBuffer.buffer[inputBuffer.fillIndex]
				
				slot.area = slot.area + area
				slot.realArea = slot.realArea + realArea
				slot.liters = slot.liters + liters
				slot.inputLiters = slot.inputLiters + liters
				slot.strawRatio = strawRatio
				slot.strawGroundType = strawGroundType
				slot.effectDensity = cutterLoad * strawRatio * 0.8 + 0.2
			end
		end
		
		if spec.fillEnableTime == nil then
			spec.fillEnableTime = g_currentMission.time + spec.processing.toggleTime
		end
		
		local pixelToSqm = g_currentMission:getFruitPixelsToSqm()
		local literPerSqm = 1
		
		if inputFruitType ~= FruitType.UNKNOWN then
			fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(inputFruitType)
			literPerSqm = fruitDesc.literPerSqm
		end
		
		local sqm = realArea * pixelToSqm
		local deltaFillLevel = sqm * literPerSqm * spec.threshingScale
		local fillType = outputFillType
		
		if spec.additives.available then
			local fillTypeSupported = false
			
			for i = 1, #spec.additives.fillTypes do
				if fillType == spec.additives.fillTypes[i] then
					fillTypeSupported = true

					break
				end
			end
			
			if fillTypeSupported then
				local additivesFillLevel = self:getFillUnitFillLevel(spec.additives.fillUnitIndex)
				
				if additivesFillLevel > 0 then
					local usage = spec.additives.usage * deltaFillLevel
					local availableUsage = usage / additivesFillLevel
					
					deltaFillLevel = deltaFillLevel * (1 + 0.05 * availableUsage)
					self:addFillUnitFillLevel(self:getOwnerFarmId(), spec.additives.fillUnitIndex, -usage, self:getFillUnitFillType(spec.additives.fillUnitIndex), ToolType.UNDEFINED)
				end
			end
		end
		
		self:setWorkedHectars(spec.workedHectars + MathUtil.areaToHa(realArea, g_currentMission:getFruitPixelsToSqm()))
		
		if farmId ~= AccessHandler.EVERYONE then
			local damage = self:getVehicleDamage()
			
			if damage > 0 then
				deltaFillLevel = deltaFillLevel * (1 - damage * Combine.DAMAGED_YIELD_REDUCTION)
			end
		end
		
		if self:getFillUnitCapacity(spec.fillUnitIndex) == math.huge and self:getFillUnitFillLevel(spec.fillUnitIndex) > 0.001 and spec.lastDischargeTime + spec.fillLevelBufferTime < g_time then
			return deltaFillLevel
		end
		
		local fillUnitIndex = spec.fillUnitIndex
		
		if spec.bufferFillUnitIndex ~= nil and self:getFillUnitFreeCapacity(spec.bufferFillUnitIndex) > 0 then
			fillUnitIndex = spec.bufferFillUnitIndex
		end
		
		if spec.loadingDelay > 0 then
			for i = 1, #spec.loadingDelaySlots do
				if not spec.loadingDelaySlots[i].valid then
					spec.loadingDelaySlots[i].valid = true
					spec.loadingDelaySlots[i].fillLevelDelta = deltaFillLevel
					spec.loadingDelaySlots[i].fillType = fillType
					
					if spec.loadingDelaySlotsDelayedInsert then
						spec.loadingDelaySlots[i].time = g_time
					else
						spec.loadingDelaySlots[i].time = g_time + spec.unloadingDelay - spec.loadingDelay
					end
					
					spec.loadingDelaySlotsDelayedInsert = not spec.loadingDelaySlotsDelayedInsert
					
					break
				end
			end
			
			return deltaFillLevel
		end
		
		local loadInfo = self:getFillVolumeLoadInfo(spec.loadInfoIndex)		
		local isNotStraw
		
		for cutter, _ in pairs(spec.attachedCutters) do
			if cutter.getIsCollectingStraw ~= nil then
				if cutter:getIsCollectingStraw() then
					local randomYield = math.random(1, 16)
					
					isNotStraw = false
					
					if randomYield == 1 then
						return self:addFillUnitFillLevel(self:getOwnerFarmId(), fillUnitIndex, deltaFillLevel, fillType, ToolType.UNDEFINED, loadInfo)
					else
						return 0
					end
				else
					isNotStraw = true
				end
			end
		end
		
		if isNotStraw then
			return self:addFillUnitFillLevel(self:getOwnerFarmId(), fillUnitIndex, deltaFillLevel, fillType, ToolType.UNDEFINED, loadInfo)
		end
	end
	
	return 0
end

function Cutter:getIsCollectingStraw()
	return self.spec_cutter.useWindrow
end